(function ($)
{
    Drupal.behaviors.openlayers = {
        attach: function (context, settings) 
        {
            console.log('erster Aufruf von OL:');
            console.log(settings.openlayers);
            $.each(settings.openlayers, function (m, data) 
            {
                $('#' + data.mapId, context).each(function () 
                {
                    var container = $(this);
                    container.data('openlayers',new Drupal.OpenLayers($(this), data.mapId, data.map, data.features));
                });
            });
        }
    };
  
    Drupal.OpenLayers = function(container, mapId, map_definition,features) {
        this.container = container;
        this.mapId = mapId;
        this.map_definition = map_definition;
        this.settings = this.map_definition.settings;
        this.bounds = [];
        this.layers = this.map_definition.layers;
        this.sources = this.map_definition.sources;
        this.navbar = this.map_definition.navbar;
        this.featureCollection = [];
        this.overlaySource = new ol.source.Vector({});
        if (features !== null && features.length > 0) {
            wktfeatures = this.createWKTFeatures(features);
            this.overlaySource.addFeatures(wktfeatures);
        }
        this.styles = {};
        this.olMap = null;
        this.controls = []; //als Controlbar
        this.initialise();
    }
    
    Drupal.OpenLayers.prototype.initialise = function() {
        this.olMap = new ol.Map({
            layers: [],
            controls:this.controls,
            target: this.mapId,
            view: new ol.View({
                center: [0, 0],
                zoom: 2
            })
        });
        if(Object.keys(this.sources).length >= 0 && Object.keys(this.layers).length >= 0) {
            this.addLayer();
        }
        if(this.featureCollection.length >= 0) {
            this.addFeatureOverlay();
        };
        if(this.navbar.controls.length > 0) {
           this.addOLControls();
        }
    };
    
    Drupal.OpenLayers.prototype.addOLControls = function() {
        for (var id in this.navbar.controls) {
            this.olMap.addControl(new window['ol'][this.navbar.controls[id]['namespace']][this.navbar.controls[id]['machine']]());
        }
    }
    /*
     * function to add all defined layers to the map
     * @returns nothing
     */
    Drupal.OpenLayers.prototype.addLayer = function() {   
        for (var layer in this.layers) {
            this.tmp = this.layers[layer];
            switch (this.layers[layer].type) {
                case 'tile':
                    this.olMap.addLayer(
                        new ol.layer.Tile({
                            source: this.addSource(),
                            opacity: this.layers[layer].opacity,
                            isBase: this.layers[layer].isBase,
                            visible:this.layers[layer].isActive,
                            id: this.layers[layer].id,
                            title:this.layers[layer].title,
                            layernames:this.layers[layer].layer,
                        })
                    )
                    break;
                case 'image':
                    this.olMap.addLayer(
                        new ol.layer.Image({
                            source: this.addSource(),
                            opacity: this.layers[layer].opacity,
                            isBase: this.layers[layer].isBase,
                            visible:this.layers[layer].isActive,
                            id: this.layers[layer].id,
                            title:this.layers[layer].title,
                            layernames:this.layers[layer].layer,
                        })
                    )
                    break;
                case 'view':
                case 'node':
                    this.olMap.addLayer(
                        new ol.layer.Vector({
                            source: this.addSource(this.layers[layer].features),
                            opacity: this.layers[layer].opacity,
                            isBase: this.layers[layer].isBase,
                            visible:this.layers[layer].isActive,
                            id: this.layers[layer].id,
                            title:this.layers[layer].title,
                            layernames:this.layers[layer].layer,
                        })
                    );
            }
        };
    };
    
    /*
     * function to add layer sources
     * @param {type} features
     * @returns {ol.source.XYZ|ol.source.ImageWMS|ol.source.Vector|ol.source.OSM}
     */
    Drupal.OpenLayers.prototype.addSource = function(features) {
        switch(this.sources[this.tmp.source].type) {
            case "osm":
                return new ol.source.OSM();
                break;
            case "imagewms":
                return new ol.source.ImageWMS({
                    url: this.sources[this.tmp.source].url,
                    params: {'LAYERS': this.tmp.layer},
                    ratio: 1,
                    serverType: this.sources[this.tmp.source].serverType,
                })
                break;
            case "xyz":
                return new ol.source.XYZ({
                    url: this.sources[this.tmp.source].url,
                });
                break;
            case "vector":
                return new ol.source.Vector({
                    features: this.createWKTFeatures(features),
                });
                break;
        }
        return null;
    }
    
    Drupal.OpenLayers.prototype.addFeatureOverlay = function() {
      var featureOverlay = new ol.layer.Vector({
        source: this.overlaySource,
      });
      featureOverlay.setMap(this.olMap);
    };
    /*
     * change every field presentation to this mod is necessary
     */
    Drupal.OpenLayers.prototype.createWKTFeatures = function(features) {
      var format = new ol.format.WKT();
      wktfeatures = [];
      for (i = 0; i < features.length; i++) {
        var feature = format.readFeature(features[i], {
          dataProjection: 'EPSG:4326',
          featureProjection: 'EPSG:3857'
        });
        wktfeatures.push(feature);
      }
      return wktfeatures;
    }
    
})(jQuery);