<?php

namespace Drupal\openlayers\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'geofield_OpenLayersInputWidget' widget.
 *
 * @FieldWidget(
 *   id = "geofield_OpenLayersInputWidget",
 *   label = @Translation("OpenLayers"),
 *   field_types = {
 *     "geofield"
 *   }
 * )
 */
class OpenLayersInputWidget extends WidgetBase {

  /**
   * Gives a visual Option to save Geometries.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $id = uniqid('', TRUE);
    $settings = [
      'mapid' => "openlayers-inputmap-" . $id,
      'selectid' => "openlayers-selectgeom",
      'wktboxid' => "openlayers-geomaswkt",
    ];

    $element['selectGeom'] = [
      '#type' => 'select',
      '#title' => $this->t('Type of Geom'),
      '#attributes' => ['id' => $settings['selectid']],
      '#options' => [
        'Point' => 'Point',
        'LineString' => 'Line',
        'Polygon' => 'Polygon',
    // 'MultiPoint' => 'Multipoint',
    // 'MultiLineString' => 'Multiline',
    // 'MultiPolygon' => 'Multipolygon',.
      ],
    ];

    $prefix = '<div class="field-geofield-openlayers-input" style="height:400px;width:400px;" id="' . $settings['mapid'] . '"></div>';

    $element['value'] = [
      '#prefix' => $prefix,
      '#type' => 'textarea',
      '#title' => $this->t('Geom as WKT'),
      '#default_value' => $items[$delta]->value ?: NULL,
      '#attributes' => ['id' => $settings['wktboxid']],
      '#attached' => [
        'library' => ['openlayers/openlayers-drupal-input'],
        'drupalSettings' => [
          'openlayers' => $settings,
        ],
      ],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $geophp = \Drupal::service('geofield.geophp');
    foreach ($values as $delta => $value) {
      if ($geom = $geophp->load($value['value'])) {
        $values[$delta]['value'] = $geom->out('wkt');
      }
    }

    return $values;
  }

}
